const logger = require("morgan");
const bcrypt = require("bcrypt");
const express = require("express");
const mongoose = require("mongoose");
const bodyParser = require("body-parser");
const session = require("express-session");

const MONGO_URL = "mongodb://localhost/mean";
const MONGO_USERNAME = "matchMaker";
const MONGO_PASSWORD = "p@ssw0rd";

//disable find and modify for findoneandupdate
mongoose.set('useFindAndModify', false);

mongoose.connect(MONGO_URL, {
    auth: {
        user: MONGO_USERNAME,
        password: MONGO_PASSWORD
    },
    useNewUrlParser: true
});
mongoose.set("useCreateIndex", true);
let meanSchema = require("./mean_schema.js").meanSchema;
//hashing a password before saving it to the database
meanSchema.pre('save', function (next) {
    bcrypt.hash(newUser.password, 10, function (err, hash){
        if (err) {
            return next(err);
        }
        newUser.password = hash;
        next();
    })
});
const User = mongoose.model("User", meanSchema);
let newUser = new User();

mongoose.connection.once("open", function () {
    console.log("Open connection!");
});


var app = express();
// Logging
app.use("/", logger("dev"));
// Static files
app.use("/static", express.static("static"));
// Body parser
app.use(bodyParser.urlencoded({extended: true}));
app.use(bodyParser.json());
//use sessions for tracking logins
app.use(session({
    secret: "it is a secret to everybody",
    resave: true,
    saveUninitialized: false
}));

// Sending those who type "localhost" to
// "localhost/static/html/register.html"
app.get("/", function (req, res) {
    res.redirect("/static/HTML/index.html");
});

// User is registering
app.post("/save/registration", function (req, res) {
    console.log("Saving Registration Information");
    console.log(req.body);
    newUser.userName = req.body.username;
    newUser.password = req.body.password;
    console.log(newUser);

    newUser.save({},function (err, doc) {
        if (err) {
            console.log(err);
            res.writeHead(500);
            res.end(JSON.stringify(err));
        } else {
            console.log("\nSaved document: " + doc);
            res.writeHead(200);
            res.end(JSON.stringify(doc));
        }
    });
});

// User is logging in
app.post("/login", function (req, res) {
    console.log("Checking Login Credentials");
    console.log(req.body);

        User.findOne({userName: req.body.username}, function(err, user) {
            //return 412 if user doesnt exist
            if(user==null){
                console.log("user doesnt exist");
                res.writeHead(412);
                res.end(JSON.stringify(err));
            }else{
                //if user exists compare passwords
                bcrypt.compare(req.body.password, user.password, function (err, doc) {
                    //if correct log user in
                    if (doc === true) {
                        console.log("valid login");
                        req.session.userId = user._id;
                        res.writeHead(200);
                        res.end(JSON.stringify(doc));
                    } else {
                        console.log("incorrect credentials");
                        res.writeHead(412);
                        res.end(JSON.stringify(err));
                    }
            });
        }
    });
});

// User is using profile page
app.post("/profilePage", function (req, res) {
    console.log("Checking Profile Page Info");
    console.log(req.body);

    //update database with profile information
    User.findOneAndUpdate({userName: req.body.username}, {$set: {firstName: req.body.firstname,
                lastName: req.body.lastname, state: req.body.state, interests: req.body.interests, profileImage: req.body.picture}},
        {upsert: true}, function(err, doc){
            if (err) {
                console.log("Something wrong when updating data!");
                res.writeHead(412);
                res.end(JSON.stringify(err));
            }
            else {
                console.log("succesfully saved");
                res.writeHead(200);
                res.end(JSON.stringify(doc));

            }
        });

});

// search page
app.post("/search", function (req, res) {
    console.log("Checking search query");
    console.log(req.body);

    //check type of search
    if(req.body.state === undefined) {
        //interest based search
        User.find({interests: req.body.interests}, function (err, doc) {
            //error check
            if(err){
                console.log("Search Failed");
                res.writeHead(412);
                res.end(JSON.stringify(err));
            }
            console.log("Interest search results" + doc);
            //send results back
            res.writeHead(200);
            res.end(JSON.stringify(doc));
        });
    }else{
        //location based search
        User.find({state: req.body.state}, function (err, doc) {
            //error check
            if(err){
                console.log("Search Failed");
                res.writeHead(412);
                res.end(JSON.stringify(err));
            }
            console.log("Location search results" + doc);
            //send results back
            res.writeHead(200);
            res.end(JSON.stringify(doc));
        });
    }

});

// Get User Info from Session
app.get("/user", function (req, res) {
    console.log("Getting _id from the session.");
    console.log(req.session.userId);

    if (req.session.userId) {
        User.findOne({ _id: req.session.userId })
            .exec(function (err, user) {
                if (err) {
                    console.log(err);
                    res.writeHead(400);
                    res.end(JSON.stringify(err));
                } else if (!user) {
                    var err = new Error("User not found.");
                    console.log(err);
                    res.writeHead(400);
                    res.end(JSON.stringify(err));
                } else {
                    console.log(user);
                    res.writeHead(200);
                    res.end(JSON.stringify(user));
                }
            });
    } else {
        var err = new Error("You must be logged in to view this page.");
        console.log(err);
        res.writeHead(401);
        res.end(JSON.stringify({message:"You must be logged in to view this page."}));
    }
});

// User is logging out
app.get("/logout", function(req, res, next) {
    if (req.session) {
        console.log("Deleting the session");
        req.session.destroy(function(err) {
            if(err) {
                return next(err);
            } else {
                res.writeHead(200);
                res.end();
                //return res.redirect("/");
            }
        });
    }
});

// Listening to port 80
app.listen(80);
